@@ -1,7 +1,7 @@
-import asyncio
-import itertools
 from typing import Tuple, Dict, List, Iterator, AsyncIterator, Optional
 
+from asyncio_throttle import Throttler
+
 from aioetherscan.exceptions import EtherscanClientApiError
 
 
@@ -15,7 +15,7 @@ class Utils:
             self,
             address: str = None,
             contract_address: str = None,
-            be_polite: bool = True,
+            throttler: Throttler = None,
             block_limit: int = 50,
             offset: int = 3,
             start_block: int = 0,
@@ -24,31 +24,19 @@ class Utils:
         if end_block is None:
             end_block = int(await self._client.proxy.block_number(), 16)
 
-        if be_polite:
-            for sblock, eblock in self._generate_intervals(start_block, end_block, block_limit):
-                result = await self._parse_by_pages(
+        if throttler is None:
+            throttler = Throttler(rate_limit=5, period=1.0)
+
+        for sblock, eblock in self._generate_intervals(start_block, end_block, block_limit):
+            async for transfer in self._parse_by_pages(
                     address=address,
                     contract_address=contract_address,
                     start_block=sblock,
                     end_block=eblock,
-                    offset=offset
-                )
-                for t in result:
-                    yield t
-        else:
-            tasks = [
-                self._parse_by_pages(
-                    address=address,
-                    contract_address=contract_address,
-                    start_block=sblock,
-                    end_block=eblock,
-                    offset=offset
-                )
-                for sblock, eblock in self._generate_intervals(start_block, end_block, block_limit)
-            ]
-            result = await asyncio.gather(*tasks)
-            for t in itertools.chain.from_iterable(result):
-                yield t
+                    offset=offset,
+                    throttler=throttler
+            ):
+                yield transfer
 
     async def token_transfers(
             self,
@@ -59,6 +47,7 @@ class Utils:
             offset: int = 3,
             start_block: int = 0,
             end_block: int = None,
+            throttler: Throttler = None,
     ) -> List[Dict]:
         kwargs = {k: v for k, v in locals().items() if k != 'self' and not k.startswith('_')}
         return [t async for t in self.token_transfers_generator(**kwargs)]
@@ -106,29 +95,31 @@ class Utils:
             start_block: int,
             end_block: int,
             offset: int,
+            throttler: Throttler = None,
             address: str = None,
             contract_address: str = None,
-    ) -> List[Dict]:
-        page, result = 1, []
-        while True:
-            try:
-                transfers = await self._client.account.token_transfers(
-                    address=address,
-                    contract_address=contract_address,
-                    start_block=start_block,
-                    end_block=end_block,
-                    page=page,
-                    offset=offset
-                )
-            except EtherscanClientApiError as e:
-                if e.message == 'No transactions found':
-                    break
-                raise
-            else:
-                result.extend(transfers)
-                page += 1
+    ) -> AsyncIterator[Dict]:
+        page = 1
 
-        return result
+        while True:
+            async with throttler:
+                try:
+                    transfers = await self._client.account.token_transfers(
+                        address=address,
+                        contract_address=contract_address,
+                        start_block=start_block,
+                        end_block=end_block,
+                        page=page,
+                        offset=offset
+                    )
+                except EtherscanClientApiError as e:
+                    if e.message == 'No transactions found':
+                        break
+                    raise
+                else:
+                    for transfer in transfers:
+                        yield transfer
+                    page += 1
 
     @staticmethod
     def _generate_intervals(from_number: int, to_number: int, count: int) -> Iterator[Tuple[int, int]]:
